/******************************************************************************
 NetBurner simple UDP send/receive example.

 This example program will allow you to send and receive UDP packets to 
 another device or host computer. To run the example, connect a serial 
 port to the debug serial port on your NetBurner device and run a dumb
 terminal program such as MTTTY. 
 
 You will be prompted for the port number to send/receive data and the 
 destination IP address of the other device or host. Note that the application 
 uses the same port number to send and receive data, but you can change this
 to any other port number you wish.

 The application will create a thread to recive packets and display them
 on the debug port, while the main task will take any data you type in
 to the MTTTY dumb terminal and send it as a udp packet to the destination
 IP address.

 *****************************************************************************/
#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <ucos.h>
#include <udp.h>
#include <autoupdate.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <netinterface.h>
#include <taskmon.h>

extern "C" {
	void UserMain(void * pd);
}

// Allocate task stack for UDP listen task
DWORD   UdpTestStk[USER_TASK_STK_SIZE];



/*-------------------------------------------------------------------
 UDP Server task will wait for incoming packets on the 
 designated port number, which is passed as a OSTaskCreate()
 void * parameter.
 -------------------------------------------------------------------*/
void UdpReaderMain(void * pd)
{
	int port = (int)pd;	  // cast void * param as int port number
	iprintf("Listening on port: %d\r\n", port);

	// Create a FIFO for the UDP packet and initialize it
	OS_FIFO fifo;
	OSFifoInit(&fifo);

	// Register to listen for UDP packets on port number 'port'
	RegisterUDPFifo(port, &fifo);

	while (1)
	{
		// Construct a UDP packet object using the previously
		// declared FIFO. The UDP constructor will only return 
		// when a packet has been received. The second parameter
		// is a timeout value (timeticks). A value of 0 will 
		// wait forever. The TICKS_PER_SECOND definition can
		// be used for code readability.
		UDPPacket upkt(&fifo, 0 * TICKS_PER_SECOND);

		// Did we get a valid packet, or just time out?
		if (upkt.Validate())
		{
			WORD len = upkt.GetDataSize();
			iprintf("Received UDP packet with %d bytes from: ", (int)len);
			ShowIP(upkt.GetSourceAddress());	// show ip address 
			iprintf("\r\n");

			ShowData(upkt.GetDataBuffer(), len); // hex dump function
			iprintf("\r\n");
		}
	}
}



/*-------------------------------------------------------------------
 UserMain
 -------------------------------------------------------------------*/
void UserMain(void * pd)
{
	int 	portnum;
	IPADDR 	ipaddr;
	char 	buffer[80];

	InitializeStack();
	
	int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf( "\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
      iprintf("Static IP Address: ");
   ShowIP(ib->netIP); iprintf("\r\n");
	

   OSChangePrio(MAIN_PRIO);
  	EnableAutoUpdate(); 
   EnableTaskMonitor();

	iprintf("Starting NetBurner UDP Example \r\n");
	iprintf("Enter the port number: ");
	scanf("%d", &portnum);

	iprintf("\r\nEnter the destination IP Address: ");
	buffer[0] = 0;
	while(buffer[0] == 0)
	{
		gets(buffer);
	}
	ipaddr = AsciiToIp(buffer);

	iprintf("Listening/Sending on Port %d, Sending to IP address: ", portnum);
	ShowIP(ipaddr);
	iprintf("\r\n");

	// Create UDP listen task
	OSTaskCreate(UdpReaderMain,
				(void  *)portnum,
				&UdpTestStk[USER_TASK_STK_SIZE] ,
				UdpTestStk,
				MAIN_PRIO - 1);	// higher priority than send task


	// Main while loop will take any user input and send it as a
	// UDP datagram. 
	while(1)
	{
		gets(buffer);
		iprintf("Sent \"%s\" to ", buffer);
		ShowIP(ipaddr);
		iprintf(":%d\r\n", portnum);
		{
			UDPPacket pkt;
			pkt.SetSourcePort(portnum);
			pkt.SetDestinationPort(portnum);
			pkt.AddData(buffer);
			pkt.AddDataByte(0);
			pkt.Send(ipaddr);
		}
		iprintf("\r\n");
	}
}




